from SPdbUSBm import SPdbUSBm
import ctypes
from ctypes import c_long, c_double, c_short, POINTER
import tkinter as tk
import subprocess
from tkinter import messagebox
from tkinter import ttk
import os
import sys
import shutil
import winreg
try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
    import matplotlib.pyplot as plt
except ImportError:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "matplotlib"])
    import matplotlib.pyplot as plt
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg


class SDK_USBm(tk.Tk):
    def __init__(self, root):


        self.line_visibility = {"Reference": True, "Real-time": True}
        self.trigger_params = {
            "bExtTrgMode": False,
            "lTrgMode": 0,  # Falling
            "lTrgTimeOut": 1,
            "bTrgTimeOutMode": False,
            "bOldTrgMode": False
        }

        self.current_dir = os.path.dirname(os.path.abspath(__file__))
        python_exe_path = os.path.dirname(sys.executable)
        print("Python execution path:", python_exe_path)
        # Full path of the DLL file
        source_file = os.path.join(self.current_dir, 'ArrayBoardR6_Lib_x64.dll')
        # Full path of the destination file to copy
        destination_file = os.path.join(python_exe_path, 'ArrayBoardR6_Lib_x64.dll')
        sdk_script_path = os.path.join(self.current_dir, '_301SDK.py')

        if not os.path.exists(destination_file):
            if not self.is_admin():
                bat_file_path = os.path.join(self.current_dir, 'run_as_admin.bat')
                subprocess.run([bat_file_path, sdk_script_path], shell=True)  
                sys.exit()  
            shutil.copy(source_file, destination_file)
            print(f"{source_file} -> {destination_file} file copy.")
        else:
            print(f"{destination_file} The file already exists. Skip copying.")

        self.root = root
        self.root.title("SDK_SM301")
        self.root.geometry("500x550")

        self.current_dir = os.path.dirname(os.path.abspath(__file__))
        dll_path = os.path.join(self.current_dir, 'SPdbUSBABR6.dll')
        self.spdb = SPdbUSBm(dll_path)
        
        self.m_strWorkStatus = None
        self.timer_id = None
        self.m_lTEC_Stability = None
        self.m_dTECTemp = ""
        self.m_lWell = 0
        self.m_bShutterPos = True
        self.is_updating = False
        self.fig = None
        self.ax = None

        # Test Board and Ready Label
        self.btn_test_board = tk.Button(self.root, text="Test Board", command=self.InitTestBorad)

        self.btn_test_board.place(x=10, y=10, width=100, height=25)

        self.lbl_ready = tk.Label(self.root, text="Ready...",relief="sunken")
        self.lbl_ready.place(x=120, y=10, width=250, height=25)

        # TE Control frame
        self.frame_te_control = tk.LabelFrame(self.root, text="TE Control")
        self.frame_te_control.place(x=10, y=50, width=480, height=130)

        self.te_cooler_var = tk.IntVar()

        self.chk_te_cooler = tk.Checkbutton(self.frame_te_control, text="TE Cooler Set", variable=self.te_cooler_var, command=self.on_te_cooler_toggle)
        self.chk_te_cooler.place(x=10, y=5)

        self.te_cooler_status = tk.Label(self.frame_te_control,relief="sunken",anchor="nw")
        self.te_cooler_status.place(x=120, y=5, width=250)

        self.lbl_static = tk.Label(self.frame_te_control, text="", relief="sunken",anchor="n", font=("Arial",8))
        self.lbl_static.place(x=230, y=35,width=200, height = 70)

        self.lbl_te_temp = tk.Label(self.frame_te_control, text="TE Cooler Temp:")
        self.lbl_te_temp.place(x=10, y=35)

        self.te_temp = tk.Entry(self.frame_te_control)
        self.te_temp.place(x=120, y=35, width=100)

        self.te_temp.bind("<KeyRelease>", self.on_change_edit_coolertemp)

        self.btn_read_tec_set_pt = tk.Button(self.frame_te_control, text="Read TEC Set PT",command=self.on_btn_read_tec_set_pt)
        self.btn_read_tec_set_pt.place(x=120, y=60,height = 20)

        # Device Setting frame
        self.frame_device_setting = tk.LabelFrame(self.root, text="Device Setting")
        self.frame_device_setting.place(x=10, y=180, width=480, height=120)

        self.lbl_int_time = tk.Label(self.frame_device_setting, text="Integration Time:")
        self.lbl_int_time.place(x=10, y=5)

        self.int_time = tk.Entry(self.frame_device_setting)
        self.int_time.insert(0, "30")
        self.int_time.place(x=120, y=5, width=100)
        self.int_time.bind("<KeyRelease>", self.change_edit_intTime)

        self.lbl_int_time_formula = tk.Label(self.frame_device_setting, text="3.2usX(intTime-1) + 4.025us")
        self.lbl_int_time_formula.place(x=230, y=5)

        self.lbl_well_size = tk.Label(self.frame_device_setting, text="Charge Well Size:")
        self.lbl_well_size.place(x=10, y=35)

        self.combo_well_size = ttk.Combobox(self.frame_device_setting, values=["1pF", "4pF", "7pF", "10pF", "11pF", "14pF", "17pF", "20pF"], state='readonly')
        self.combo_well_size.set("1pF")
        self.combo_well_size.place(x=120, y=35, width=100)

        self.combo_well_size.bind("<<ComboboxSelected>>", self.on_selchange_combo_well)

        self.lbl_time_ave = tk.Label(self.frame_device_setting, text="Time Averaging:")
        self.lbl_time_ave.place(x=230, y=35)

        self.time_ave = tk.Entry(self.frame_device_setting)
        self.time_ave.insert(0, "1")
        self.time_ave.place(x=330, y=35, width=100)

        self.btn_balance = tk.Button(self.frame_device_setting, text="Balance",state="disabled", command = self.btn_balance)
        self.btn_balance.place(x=10, y=65, width=100)

        self.btn_shutter = tk.Button(self.frame_device_setting, text="Shutter Position", command=self.on_btn_shutter)
        self.btn_shutter.place(x=120, y=65, width=100)

        self.btn_trigger = tk.Button(self.frame_device_setting, text="Trigger Mode", command= self.open_trigger_mode)
        self.btn_trigger.place(x=230, y=65, width=100)

        # Get Data from CCD frame
        self.frame_get_data = tk.LabelFrame(self.root, text="Get Data from CCD")
        self.frame_get_data.place(x=10, y=310, width=480, height=100)

        self.radio_var = tk.IntVar()
        self.radio_var.set(1)  # Set initial value (the first button is selected by default)

        self.rd_raw_data = tk.Radiobutton(self.frame_get_data, text="Raw Data", variable=self.radio_var, value=1, command =self.radio_select)
        self.rd_raw_data.place(x=10, y=5)

        # Baseline Correction Radiobutton
        self.rd_baseline = tk.Radiobutton(self.frame_get_data, text="Baseline Correction", variable=self.radio_var, value=2, command =self.radio_select)
        self.rd_baseline.place(x=150, y=5)

        self.lbl_gain = tk.Label(self.frame_get_data, text="Gain:")
        self.lbl_gain.place(x=10, y=35)

        self.gain = tk.Entry(self.frame_get_data)
        self.gain.place(x=50, y=35, width=100)
        self.gain.insert(0,"1.5")
        self.gain.config(state='disable')

        self.lbl_offset = tk.Label(self.frame_get_data, text="Offset:")
        self.lbl_offset.place(x=160, y=35)

        self.offset = tk.Entry(self.frame_get_data,)
        self.offset.place(x=210, y=35, width=100)
        self.offset.insert(0,"7000")
        self.offset.config(state='disable')

        self.btn_get_data = tk.Button(self.frame_get_data, text="Get Data",state="disabled", command = self.btn_getData)
        self.btn_get_data.place(x=330, y=10, width=100)

        self.btn_grahp = tk.Button(self.frame_get_data, text = "Graph",state="disabled" ,command= self.graph_click)
        self.btn_grahp.place(x=330, y=40, width=100)

        # Status label
        self.lbl_status = tk.Label(self.root, text="", anchor="nw", relief="sunken")
        self.lbl_status.place(x=10, y=420, width=480, height=120)

        self.root.protocol('WM_DELETE_WINDOW',self.on_close_x)
        self.InitTestBorad()
    
    def is_admin(self):
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except:
            return False

    def InitTestBorad(self):
        sRtn = self.spdb.spTestBoard()
        if sRtn > 0:
            self.lbl_ready.config(text="The PbS array detector board is connected.")

            self.int_time.config(state="normal")
            self.te_temp.config(state="normal")
            self.btn_read_tec_set_pt.config(state="normal")
            self.combo_well_size.config(state="readonly")

            IntTime = int(self.int_time.get())

            integration_time = 3.2 * (IntTime - 1) + 4.025
            formatted_time ="{:.1f}".format(integration_time)
            self.lbl_int_time_formula.config(text="3.2usX(intTime -1)+4.025us = " + str(formatted_time) + "uSec")

            if self.spdb.spSetIniVarEx(0)<0:
                self.DisableCtrls()
                return

            lStatusTE = (ctypes.c_long * 3)()
            dTeVal = (ctypes.c_double * 4)()
            # TE Cooler setting check
            if self.spdb.spReadStatusTE(0,8,lStatusTE,dTeVal) < 0:
                self.DisableCtrls()
                return

            self.m_lTEC_PWR = lStatusTE[0]
            self.m_lTEC_Cooling = lStatusTE[1]
            self.m_lTEC_Stability = lStatusTE[2]

            self.te_cooler_var.set(bool(self.m_lTEC_PWR))

            if self.te_cooler_var.get():
                self.te_cooler_status.config(text="TE Cooler is On...")
                self.m_strWorkStatus = "Waiting for TEC stabilization check......\n"
                self.timer_id = self.root.after(500, self.onTimer)
            else:
                self.te_cooler_status.config(text="TE Cooler is Off...")

        elif sRtn == -1:
            self.lbl_ready.config(text="The PbS array detector board is NOT connected.")

    def DisableCtrls(self):
        self.int_time.config(state="disabled")
        self.te_temp.config(state="disabled")
        self.btn_read_tec_set_pt.config(state="disabled")
        self.combo_well_size.config(state="disabled")
        self.btn_get_data.config(state="disabled")
        self.btn_grahp.config(state="disabled")

    def kill_timer(self):
        if self.timer_id is not None:
            self.root.after_cancel(self.timer_id)
            self.timer_id = None

    def on_te_cooler_toggle(self):
        if self.te_cooler_var.get() == 1:
            self.te_cooler_status.config(text="TE Cooler is On...")
            if self.spdb.spTECoolerPWR(0,1) < 0:
                self.DisableCtrls()
                return
            self.m_strWorkStatus = "Waiting for TEC stabilization check......\n"
            self.timer_id = self.root.after(500, self.onTimer)
        else:
            self.te_cooler_status.config(text="TE Cooler is Off...")
            if self.spdb.spTECoolerPWR(0,0) < 0:
                if self.timer_id is not None:
                    self.kill_timer()
                    self.DisableCtrls()
                    return
            self.kill_timer()

    def onTimer(self):
        lStatusTE = (ctypes.c_long * 3)()  # Array for status
        dTeVal = (ctypes.c_double * 4)()   # Array for temperature values
        IOldStability = self.m_lTEC_Stability

        if self.spdb.spReadStatusTE(0,8,lStatusTE,dTeVal) < 0:
            self.DisableCtrls()
            return

        ITEC, TMON, VTEC, VREF = dTeVal[0], dTeVal[1], dTeVal[2], dTeVal[3]
        self.lbl_static.config(text = f"ITEC: {ITEC:.3f} A\nTMON: {TMON:.3f} mK\nVTEC: {VTEC:.3f} V\nVREF: {VREF:.3f}")

        m_lTEC_PWR = lStatusTE[0]
        m_lTEC_Cooling = lStatusTE[1]
        self.m_lTEC_Stability = lStatusTE[2]

        TEC_PWR = "ON" if m_lTEC_PWR == 1 else "OFF"
        TEC_Cooling = "Cooling" if  m_lTEC_Cooling == 1 else "Heating"

        # stability test
        if self.m_lTEC_Stability == 1:
            self.btn_balance.config(state="normal")
            TEC_Stability = "Y"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
            if IOldStability != self.m_lTEC_Stability:
                self.m_strWorkStatus = "TEC stabilization!"
        else:
            self.btn_balance.config(state="disabled")
            self.btn_get_data.config(state="disabled")
            self.btn_grahp.config(state="disabled")
            TEC_Stability = "N"
            if IOldStability != self.m_lTEC_Stability :
                self.m_strWorkStatus = "Waiting for TEC stabilization check......\n"

        if self.trigger_params["bExtTrgMode"]:
            self.btn_get_data.config(state="normal")
            self.btn_grahp.config(state="normal")

        self.m_strTECStatus = f"TEC PWR: {TEC_PWR}, TEC Cooling: {TEC_Cooling}, TEC Stability: {TEC_Stability}\n"
        self.lbl_status.config(text=self.m_strTECStatus + self.m_strWorkStatus)
        self.timer_id = self.root.after(500, self.onTimer)


    def on_change_edit_coolertemp(self, event):
        str_old_temp = self.m_dTECTemp

        # Get the current value from the entry
        self.m_dTECTemp = self.te_temp.get().strip()

        if self.m_dTECTemp != '-' and self.m_dTECTemp:
            try:
                dTECTemp = float(self.m_dTECTemp)

                if self.te_cooler_var.get():
                    # Limit the temperature
                    if dTECTemp < -21.1:
                        dTECTemp = -21.1
                    elif dTECTemp > 28.5:
                        dTECTemp = 28.5

                    # Set the temperature to the TE Cooler
                    if self.spdb.spSetTECoolerEx(0, dTECTemp) < 0:
                        self.DisableCtrls()  # Disable the GUI
                        # Add any additional actions here if necessary
                        return
                else:
                    messagebox.showwarning("Warning", "Turn on TEC power!!")
                    self.te_temp.delete(0, tk.END)  # Clear the entry
                    self.te_temp.insert(0, str_old_temp)  # Restore the previous value

            except ValueError:
                self.m_dTECTemp = str_old_temp
                messagebox.showwarning("Warning", "Turn on TEC power!!")
                return

    def on_btn_read_tec_set_pt(self):
        lTemp = ctypes.c_long()  # Use ctypes, equivalent to C++ long type
        TempC = ctypes.c_double()
        if self.spdb.spTECoolerReadTECPT(0, lTemp) < 0:
            self.DisableCtrls()
            if self.te_cooler_var.get():
                self.kill_timer()
            return
        else:
            self.spdb.spTECPT_DigToAnal(lTemp, TempC)  # Read value from lTemp
            self.te_temp.delete(0, tk.END)
            formatted_tempC = "{:.3f}".format(float(TempC.value))
            self.te_temp.insert(0, f"{formatted_tempC}")  # Format to 3 decimal places

    def on_close_x(self):
        if  self.te_cooler_var.get():
            self.kill_timer()  

        if self.trigger_params["bExtTrgMode"]:
            self.spdb.spSetExtTrgMode(0, 0)  

        self.spdb.spCloseEx()
        self.root.destroy()

    def on_selchange_combo_well(self, event):
        selected_index = self.combo_well_size.current()
        
        # Process if the selected value is different from the previous one
        if selected_index != self.m_lWell:
            self.m_lWell = selected_index

            # Call spSetWellDepth function (assuming you are calling the DLL function with ctypes)
            result = self.spdb.spSetWellDepth(0, self.m_lWell)
            if result < 0:
                self.DisableCtrls()
                if self.te_cooler_var.get():
                    self.kill_timer()
                return
            self.btn_get_data.config(state="disabled")

    def on_btn_shutter(self):
        if self.m_bShutterPos:
            self.m_bShutterPos = False
            IRtn = self.spdb.spSetShutterPos(0,0)
            if IRtn < 0:
                self.DisableCtrls()
                self.kill_timer()
                return
            self.m_strWorkStatus = "Set Shutter Position OK! Shutter Close."
        else:
            self.m_bShutterPos = True
            IRtn = self.spdb.spSetShutterPos(0,1)
            if IRtn < 0:
                self.DisableCtrls()
                self.kill_timer()
                return
            self.m_strWorkStatus = "Set Shutter Position OK! Shutter Open."

    def change_edit_intTime(self,event):
        try:
            int_time_value = int(self.int_time.get())
            int_time_c_long = ctypes.c_long(int_time_value)

            # Check range and adjust values
            if int_time_c_long.value < 1:
                int_time_c_long = ctypes.c_long(1)
                self.int_time.delete(0, tk.END)  # Clear the current value
                self.int_time.insert(0, str(int_time_c_long.value))  # Enter new value
            elif int_time_c_long.value > 65535:
                int_time_c_long = ctypes.c_long(65535)

                self.int_time.delete(0, tk.END)
                self.int_time.insert(0, str(int_time_c_long.value))

            # Set integration time and handle errors
            if self.spdb.spSetIntTimeEx(0, int_time_c_long) < 0:
                self.DisableCtrls()
                if self.te_cooler_var.get():
                    self.kill_timer()

            integration_time = 3.2 * (int_time_value - 1) + 4.025
            formatted_time ="{:.1f}".format(integration_time)
            self.lbl_int_time_formula.config(text="3.2usX(intTime -1)+4.025us = " + str(formatted_time) + "uSec")
        except ValueError:
            # Handle when a non-numeric value is entered
            messagebox.showerror("Input Error", "Please enter a valid number.")
            self.int_time.delete(0, tk.END)
            self.int_time.insert(0, "1")  

    def btn_balance(self):
        self.kill_timer()
        if self.spdb.spBalanceEx(0,0) < 0:
            self.DisableCtrls()
            self.kill_timer()
            return
        self.timer_id = self.root.after(500, self.onTimer)
        self.btn_get_data.config(state="normal")
        self.btn_grahp.config(state="normal")

    def radio_select(self):
        if self.radio_var.get() == 1:
            self.gain.config(state = "disable")
            self.offset.config(state = "disable")
        else:
            self.gain.config(state = "normal")
            self.offset.config(state = "normal")

    def open_trigger_mode(self):
        TriggerModeDialog(self.root, self.trigger_params,self.spdb)
        self.trigger_params

    def btn_getData(self):
        plArrayData = (ctypes.c_long * 256)()
        plTempData = (ctypes.c_long * 256)()

        int_gain_value = ctypes.c_double(float(self.gain.get()))

        int_offset_c_int = ctypes.c_int(int(self.offset.get()))
        ave = int(self.time_ave.get())
        for _ in range(ave):
            print(f"Loop {_} started")
            if not self.trigger_params["bExtTrgMode"]:
                if self.radio_var.get() == 1:
                    print("Reading raw data")
                    sRtn = self.spdb.spReadDataEx_Raw(0, plArrayData)
                else:
                    print("Reading advanced data")
                    sRtn = self.spdb.spReadDataEx_Adv(0, int_gain_value, int_offset_c_int, plArrayData)

                print(f"sRtn value: {sRtn}")
                if sRtn != 1:
                    print("Error: sRtn is not 1, disabling controls")
                    self.DisableCtrls()
                    return
            else:
                while True:
                    if self.radio_var.get() == 1:
                        print("Reading raw data in loop")
                        sRtn = self.spdb.spReadDataEx_Raw(0, plArrayData)
                    else:
                        print("Reading advanced data in loop")
                        sRtn = self.spdb.spReadDataEx_Adv(0, int_gain_c_double, int_offset_c_int, plArrayData)

                    print(f"sRtn value in loop: {sRtn}")
                    if sRtn != -99:
                        break
                if sRtn != 1:
                    print("Error: sRtn is not 1 in loop, disabling controls")
                    self.DisableCtrls()
                    return

            for j in range(256):
                plTempData[j] += plArrayData[j]
                print(f"Accumulating data at index {j}, plTempData[j]: {plTempData[j]}")

        for j in range(256):
            plArrayData[j] = int(plTempData[j] / ave + 0.5)
            print(f"Final averaging at index {j}, plArrayData[j]: {plArrayData[j]}")

        self.m_strWorkStatus = "The data reading was done successfully!\n\r";

        dWaveLength = (ctypes.c_double * 3)()
        dPixelNo = (ctypes.c_double * 3)()
        dWaveLength_value = [1318.0, 1550.0, 2636.0]
        dPixelNo_value = [33, 63, 206]
        coeffs = (ctypes.c_double * 5)()
        for i in range(3):
            dWaveLength[i] = dWaveLength_value[i] 
            dPixelNo[i] = dPixelNo_value[i]
        dWL = ctypes.c_double()
        self.spdb.spPolyFit(dPixelNo,dWaveLength,3,coeffs,2)
        str_file = os.path.join(self.current_dir, "SDK_SM301.txt")

        with open(str_file, 'w') as sioFile:
            sioFile.write(" Pixel\t\tWavelength(nm)\tIntensity \n")
            for i in range(256):
                self.spdb.spPolyCalc(coeffs, 2, ctypes.c_double(i+1),dWL)
                sioFile.write(f" {i}\t{dWL.value}\t{plArrayData[i]}\n")

        self.m_strWorkStatus += "The data was saved as [SDK_SM301.txt] successfully!\n\r"
        pass


    # graph draw
    def graph_click(self):
        graph_window = self.create_graph_window()

        self.fig, self.ax = plt.subplots(figsize=(10, 6))
        self.update_Data(self.fig, self.ax)

        self.add_canvas_to_window(graph_window, self.fig)
        self.add_buttons_to_window(graph_window, self.fig, self.ax)

    def create_graph_window(self):
        graph_window = tk.Toplevel(self.root)
        graph_window.title("Wavelength Intensity")
        return graph_window

    def graph_data(self):
        plArrayData = (ctypes.c_long * 256)() # Array size 1 margin
        plTempData = (ctypes.c_long * 256)() # Array size 1 margin

        int_gain_value = ctypes.c_double(float(self.gain.get()))

        int_offset_c_int = ctypes.c_int(int(self.offset.get()))
        ave = int(self.time_ave.get())

        dWaveLength = (ctypes.c_double * 3)()
        dPixelNo = (ctypes.c_double * 3)()
        dWaveLength_value = [1318.0, 1550.0, 2636.0]  # WaveLength
        dPixelNo_value = [33, 63, 206]  # Pixel
        coeffs = (ctypes.c_double * 5)()
        for i in range(3):
            dWaveLength[i] = dWaveLength_value[i] 
            dPixelNo[i] = dPixelNo_value[i]
        dWL = ctypes.c_double()
        self.spdb.spPolyFit(dPixelNo,dWaveLength,3,coeffs,2)
        dWLTable = (ctypes.c_double * 256)() # Array size 1 margin

        for _ in range(ave):
    
            if not self.trigger_params["bExtTrgMode"]:
                if self.radio_var.get() == 1:
                    sRtn = self.spdb.spReadDataEx_Raw(0, plArrayData)
                else:
                    sRtn = self.spdb.spReadDataEx_Adv(0, int_gain_value, int_offset_c_int, plArrayData)

                if sRtn != 1:
                    self.DisableCtrls()
                    return
            else:
                while True:
                    if self.radio_var.get() == 1:
                        sRtn = self.spdb.spReadDataEx_Raw(0, plArrayData)
                    else:
                        sRtn = self.spdb.spReadDataEx_Adv(0, int_gain_c_double, int_offset_c_int, plArrayData)

                    if sRtn != -99:
                        break
                if sRtn != 1:
                    self.DisableCtrls()
                    return

            for j in range(256):
                self.spdb.spPolyCalc(coeffs, 2, ctypes.c_double(j+1), dWL)
                dWLTable[j] = dWL
                plTempData[j] += plArrayData[j]

        for j in range(256):
            plArrayData[j] = int(plTempData[j] / ave + 0.5)

        return plArrayData, dWLTable

    def updating(self,fig,ax):
        if self.is_updating:
            self.update_Data(fig,ax)
            ave = int(self.time_ave.get())
            int_time_value = int(self.int_time.get())
            fig.canvas.get_tk_widget().after(int_time_value * ave, self.updating, fig, ax)
        

    def start_updating(self,fig,ax):
        if not self.is_updating:
            self.is_updating = True
            self.updating(fig,ax)

    def stop_updating(self):
        self.is_updating = False


    def update_Data(self, fig, ax):
        intensity, WLTable = self.graph_data()

        if WLTable is None or intensity is None:
            return
        try:
            ax.clear()  # Clear previous data
            real_time_line, = ax.plot(WLTable, intensity, label="Real-time", picker=True)
            real_time_line.set_visible(self.line_visibility["Real-time"])  # Maintain previous visibility state
            ax.set_title("Wavelength Intensity")
            ax.set_xlabel("Wavelength (nm)")
            ax.set_ylabel("Intensity")
            ax.grid(True)
            fig.canvas.draw()  # Redraw the graph
        except Exception as e:
            print(f"Error occurred during plotting: {e}")
        
    # Open a new window for the graph
    def add_canvas_to_window(self, graph_window, fig):
        self.is_updating = False
        canvas = FigureCanvasTkAgg(fig, master=graph_window)
        canvas.draw()
        canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=True)

    # Add buttons to the graph window
    def add_buttons_to_window(self, graph_window, fig, ax):
        button_frame = tk.Frame(graph_window)
        button_frame.pack(side=tk.BOTTOM, fill=tk.X)

        start_button = tk.Button(button_frame, text="Start", command=lambda: self.start_updating(self.fig, self.ax))
        start_button.pack(side=tk.LEFT, padx=10, pady=10)

        stop_button = tk.Button(button_frame, text="Stop", command = self.stop_updating)
        stop_button.pack(side=tk.LEFT, padx=10, pady=10)

        close_button = tk.Button(button_frame, text="Close", command=graph_window.destroy)
        close_button.pack(side=tk.RIGHT, padx=10, pady=10)

        

class TriggerModeDialog(tk.Toplevel):
    def __init__(self, parent, trigger_params,spdb):
        if isinstance(parent, tk.Tk):
            super().__init__(parent)
        else:
            raise ValueError("Parent must be an instance of tk.Tk")
        self.title("Trigger Mode")
        self.geometry("350x200")
        self.trigger_params = trigger_params
        self.trigger_params["bOldTrgMode"] = self.trigger_params["bExtTrgMode"]
        self.spdb = spdb

        # External Trigger Mode
        self.ext_trg_var = tk.BooleanVar(value=self.trigger_params["bExtTrgMode"])
        self.chk_ext_trg_mode = tk.Checkbutton(self, text="External Trigger Mode", variable=self.ext_trg_var, command = self.toggle_ext_trigger)
        self.chk_ext_trg_mode.grid(row=0, column=0, padx=10, pady=5)

        # Trigger Mode
        self.label_trg_mode = tk.Label(self, text="Trigger Mode")
        self.label_trg_mode.grid(row=1, column=0, padx=10, pady=5)
        
        self.trg_mode = ttk.Combobox(self, values=["Falling", "Rising"])
        self.trg_mode.current(self.trigger_params["lTrgMode"])
        self.trg_mode.grid(row=1, column=1, padx=10, pady=5)

        # Trigger Time Out
        self.bTrgTimeOutMode_var = tk.BooleanVar(value=self.trigger_params["bTrgTimeOutMode"])

        # Trigger Time Out On/Off Checkbutton
        self.chk_trg_timeout_mode = tk.Checkbutton(self, text="Trigger Time Out On/Off", variable=self.bTrgTimeOutMode_var)
        self.chk_trg_timeout_mode.grid(row=1, column=0)

        self.label_trg_timeout = tk.Label(self, text="Trigger Time Out")
        self.label_trg_timeout.grid(row=3, column=0, padx=10, pady=5)

        self.entry_trg_timeout = tk.Entry(self)
        self.entry_trg_timeout.insert(0, str(self.trigger_params["lTrgTimeOut"]))
        self.entry_trg_timeout.grid(row=3, column=1, padx=10, pady=5)

        # OK and Cancel buttons
        self.btn_ok = tk.Button(self, text="OK", command=self.on_ok)
        self.btn_ok.grid(row=4, column=0, padx=10, pady=10)

        self.btn_cancel = tk.Button(self, text="Cancel", command=self.destroy)
        self.btn_cancel.grid(row=4, column=1, padx=10, pady=10)

        self.toggle_ext_trigger()

    def toggle_ext_trigger(self):
        if self.ext_trg_var.get():
            self.trg_mode.config(state='normal')
            self.entry_trg_timeout.config(state='normal')
            self.chk_trg_timeout_mode.config(state='normal')
        else:
            self.trg_mode.config(state='disabled')
            self.entry_trg_timeout.config(state='disabled')
            self.chk_trg_timeout_mode.config(state='disabled')

    def on_ok(self):
        timeout = int(self.entry_trg_timeout.get())
        timeout = ctypes.c_long(timeout)
        trgmode = ctypes.c_long(self.trg_mode.current()) 
        if not self.trigger_params["bOldTrgMode"] and self.ext_trg_var.get():
            if self.entry_trg_timeout.get():
                IRtn = self.spdb.spConfgTriggerDelay(0,timeout,1)
            else:
                IRtn = self.spdb.spConfgTriggerDelay(0,1,0)

            if IRtn < 0:
                messagebox.ERROR("External Trigger Time Out Setting Error..!!")

            if self.spdb.spConfgTrigger(0,trgmode) < 0:
                messagebox.ERROR("External Trigger Mode Setting Error..!!")

            if self.spdb.spSetExtTrgMode(0,1) <0:
                messagebox.ERROR("External Trigger Setting Error..!!")
            else:
                self.trigger_params["bExtTrgMode"] = self.ext_trg_var.get()
                self.trigger_params["lTrgMode"] = self.trg_mode.current()
                self.trigger_params["lTrgTimeOut"] = int(self.entry_trg_timeout.get())
                self.trigger_params["bTrgTimeOutMode"] = self.bTrgTimeOutMode_var.get()

        if self.trigger_params["bOldTrgMode"] and not self.ext_trg_var.get():                                                                                                               
            self.spdb.spSetExtTrgMode(0,0)

            IRtn = -1
            while IRtn != 1:
                IRtn = self.spdb.spSetExtTrgMode(0,0)

            if IRtn > 0 :
                self.trigger_params["bExtTrgMode"] = self.ext_trg_var.get()
                self.trigger_params["lTrgMode"] = self.trg_mode.current()
                self.trigger_params["lTrgTimeOut"] = int(self.entry_trg_timeout.get())
                self.trigger_params["bTrgTimeOutMode"] = self.bTrgTimeOutMode_var.get()

        self.destroy()

if __name__ == "__main__":
    root = tk.Tk()
    app = SDK_USBm(root)
    root.mainloop()